#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class InventoryDeviceTypeObjectsController < ApplicationController
  before_action :set_inventory_device_type_object, only: [:edit, :update, :destroy]

  layout false, only: [:new, :edit, :import]

  # GET /inventory_device_type_objects
  # GET /inventory_device_type_objects.json
  def index
    session.delete(:search)
    @inventory_device_type_objects = InventoryDeviceTypeObject.all
                          .order(created_at: :desc)
                          .paginate(page: 1, per_page: current_user.per_page_count)
  end

  # GET /inventory_device_type_objects/new
  def new
    @inventory_device_type_object = InventoryDeviceTypeObject.new(name: params[:name])
  end

  # POST /inventory_device_type_objects
  # POST /inventory_device_type_objects.json
  def create
    @inventory_device_type_object = InventoryDeviceTypeObject.new(inventory_device_type_object_params)

    if @inventory_device_type_object.save
      if session[:search].present?
        filter = ['( LOWER(name) LIKE :search or ' \
                'LOWER(description) LIKE :search LIKE :search)',
                search: "%#{session[:search].try(:downcase)}%"]
        @inventory_device_type_objects = InventoryDeviceTypeObject.where(filter)
      else
        @inventory_device_type_objects = InventoryDeviceTypeObject.all
      end
      new_page = params[:page] ? params[:page].to_i : 1
      @inventory_device_type_objects = @inventory_device_type_objects.order(created_at: :desc)
                            .paginate(page: new_page, per_page: current_user.per_page_count)
      flash.now[:notice] = I18n.t('inventory_device_type_objects.create.success')
    else
      flash[:alert] = I18n.t('inventory_device_type_objects.create.error')
    end
  end

  # PATCH/PUT /inventory_device_type_objects/1
  # PATCH/PUT /inventory_device_type_objects/1.json
  def update
    if @inventory_device_type_object.update(inventory_device_type_object_params)
      flash.now[:notice] = I18n.t('inventory_device_type_objects.update.success')
    else
      flash[:alert] = I18n.t('inventory_device_type_objects.update.error')
    end
  end

  # DELETE /inventory_device_type_objects/1
  # DELETE /inventory_device_type_objects/1.json
  def destroy
    if @inventory_device_type_object && @inventory_device_type_object.destroy
      # Keep filter when deleting
      if session[:search].present?
        filter = ['( LOWER(name) LIKE :search or ' \
                'LOWER(description) LIKE :search LIKE :search)',
                search: "%#{session[:search].try(:downcase)}%"]
        @inventory_device_type_objects = InventoryDeviceTypeObject.where(filter)
      else
        @inventory_device_type_objects = InventoryDeviceTypeObject.all
      end

      # Keep page in range and deleting last page if it is empty
      current_page = [params[:page].to_i, 1].max
      max_page = (@inventory_device_type_objects.any? ? (@inventory_device_type_objects.count.to_f / current_user.per_page_count).ceil : 1)
      new_page = [current_page, max_page].min

      @inventory_device_type_objects = @inventory_device_type_objects
                            .order(created_at: :desc)
                            .paginate(page: new_page, per_page: current_user.per_page_count)
      flash.now[:notice] = I18n.t('inventory_device_type_objects.destroy.success')
    else
      flash[:alert] = I18n.t('inventory_device_type_objects.destroy.error')
    end
  end

  #
  # Search in applications
  #
  def search
    if params[:search].blank?
      session.delete(:search)
      @inventory_device_type_objects = InventoryDeviceTypeObject.all
                          .order(created_at: :desc)
                          .paginate(page: params[:page], per_page: current_user.per_page_count)
    else
      session[:search] = params[:search] if params[:search].present?
      filter = ['( LOWER(name) LIKE :search or ' \
              'LOWER(description) LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]
      @inventory_device_type_objects = InventoryDeviceTypeObject.where(filter)
                            .order(created_at: :desc)
                            .paginate(page: params[:page], per_page: current_user.per_page_count)
    end
  end

  #
  # Export all applications as CSV
  #
  def export
    filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') + '-Applications.csv' }
    filter = ['( LOWER(name) LIKE :search or ' \
              'LOWER(description) LIKE :search LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]
    content = InventoryDeviceTypeObject.where(filter).order(:name)
              .map { |o| "#{o.name};#{o.description}" }.join("\n")
    send_data content, filename: filename
  end

  #
  # Import applications from a given CSV file.
  #
  def import_file
    content_file = params[:inventory_device_type_object][:csv_file]

    if params[:add_vars] == 'true'
      count = InventoryDeviceTypeObject.add_apps_from_file(content_file)
      msg = "#{count[0]} #{I18n.t('controllers.added')} #{count[1]} #{I18n.t('controllers.updated')}"
    else
      count = InventoryDeviceTypeObject.replace_apps_from_file(content_file)
      msg = "#{count} #{I18n.t('controllers.added')}"
    end

    redirect_to objects_path, notice: msg
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_inventory_device_type_object
    @inventory_device_type_object = InventoryDeviceTypeObject.find_by(id: params[:id])
    return if @inventory_device_type_object
    # The object doesn't exist!
    rb_flash "#{I18n.t('controllers.not_exist')}", type: :error
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def inventory_device_type_object_params
    params.require(:inventory_device_type_object).permit(:name, :description, :page, :search)
  end
end
